using System;
using DeltaEngine.Content;
using DeltaEngine.Core;
using DeltaEngine.Datatypes;
using DeltaEngine.Entities;
using DeltaEngine.Rendering2D;
using DeltaEngine.Rendering2D.Particles;

namespace $safeprojectname$
{
	public class InteractionLogics : Entity, Updateable
	{
		public InteractionLogics()
		{
			explosionData = ContentLoader.Load<ParticleEmitterData>("ExplosionEffectEmitter0");
			IncreaseScore += i => { };
		}

		public void BeginGame()
		{
			gameRunning = true;
			Player = new PlayerShip();
		}

		public PlayerShip Player { get; private set; }
		private ParticleEmitterData explosionData;
		private bool gameRunning;

		public void CreateRandom$safeprojectname$(int howMany, int sizeMod = 1)
		{
			for (int asteroidCount = 0; asteroidCount < howMany; asteroidCount++)
			{
				new Asteroid(this, sizeMod);
			}
		}

		public void Create$safeprojectname$AtPosition(Vector2D position, int sizeMod = 1, int howMany = 2)
		{
			for (int asteroidCount = 0; asteroidCount < howMany; asteroidCount++)
			{
				var asteroid = new Asteroid(this, sizeMod);
				asteroid.SetWithoutInterpolation(new Rectangle(position, asteroid.DrawArea.Size));
			}
		}

		public void IncrementScore(int increase)
		{
			IncreaseScore.Invoke(increase);
		}

		private void CheckAsteroidCollisions()
		{
			foreach (var asteroid in EntitiesRunner.Current.GetEntitiesOfType<Asteroid>())
			{
				//ncrunch: no coverage start
				foreach (var projectile in EntitiesRunner.Current.GetEntitiesOfType<Projectile>())
					if (ObjectsInHitRadius(projectile, asteroid, 0.1f / asteroid.sizeModifier))
					{
						var explosionEmitter = new ParticleEmitter(explosionData, new Vector3D(projectile.Center));
						explosionEmitter.RenderLayer = 10;
						explosionEmitter.DisposeAfterSeconds(0.7f);
						projectile.Dispose();
						asteroid.Fracture();
					}
				//ncrunch: no coverage end
				if (Player.IsActive && ObjectsInHitRadius(Player, asteroid, 0.06f / asteroid.sizeModifier))
				{
					Player.IsActive = false;
					var explosionEmitter = new ParticleEmitter(explosionData, new Vector3D(Player.Center));
					explosionEmitter.RenderLayer = 10;
					explosionEmitter.DisposeAfterSeconds(0.5f);
					if(GameOver != null)
						GameOver();
				}
			}
		}

		private static bool ObjectsInHitRadius(Entity2D entityAlpha, Entity2D entitytBeta,
			float radius)
		{
			return entityAlpha.DrawArea.Center.DistanceTo(entitytBeta.DrawArea.Center) < radius;
		}

		private void CreateNewAsteroidIfNecessary()
		{
			if (GlobalTime.Current.Milliseconds - 1000 > timeLastNewAsteroid &&
				EntitiesRunner.Current.GetEntitiesOfType<Asteroid>().Count <= Maximum$safeprojectname$)
			{
				CreateRandom$safeprojectname$(1);
				timeLastNewAsteroid = GlobalTime.Current.Milliseconds;
			}
		}

		private const int Maximum$safeprojectname$ = 10;
		private float timeLastNewAsteroid;
		public event Action GameOver;
		public event Action<int> IncreaseScore;

		public void Restart()
		{
			DisposeObjects();
			BeginGame();
			gameRunning = true;
		}

		public void DisposeObjects()
		{
			foreach (Asteroid asteroid in EntitiesRunner.Current.GetEntitiesOfType<Asteroid>())
				asteroid.IsActive = false; //ncrunch: no coverage 
			foreach (Projectile projectile in EntitiesRunner.Current.GetEntitiesOfType<Projectile>())
				projectile.Dispose(); //ncrunch: no coverage
			foreach (PlayerShip playerShip in EntitiesRunner.Current.GetEntitiesOfType<PlayerShip>())
				playerShip.IsActive = false;
		}

		public void Update()
		{
			if(!gameRunning)
				return;
			CheckAsteroidCollisions();
			CreateNewAsteroidIfNecessary();
		}

		public void PauseUpdate()
		{
			gameRunning = false;
		}

		public bool IsPauseable { get { return true; } }
	}
}